"""
    
    Summary product unit tests
    
    Copyright 2007 Red Innovation http://www.redinnovation.com
    
"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'plaintext'

from Testing import ZopeTestCase

# Make the boring stuff load quietly
ZopeTestCase.installProduct('CMFCore', quiet=1)
ZopeTestCase.installProduct('CMFDefault', quiet=1)
ZopeTestCase.installProduct('CMFCalendar', quiet=1)
ZopeTestCase.installProduct('CMFTopic', quiet=1)
ZopeTestCase.installProduct('DCWorkflow', quiet=1)
ZopeTestCase.installProduct('CMFActionIcons', quiet=1)
ZopeTestCase.installProduct('CMFQuickInstallerTool', quiet=1)
ZopeTestCase.installProduct('CMFFormController', quiet=1)
ZopeTestCase.installProduct('GroupUserFolder', quiet=1)
ZopeTestCase.installProduct('ZCTextIndex', quiet=1)
ZopeTestCase.installProduct('TextIndexNG2', quiet=1)
ZopeTestCase.installProduct('SecureMailHost', quiet=1)
ZopeTestCase.installProduct('CMFPlone', quiet=1)
ZopeTestCase.installProduct('Archetypes', quiet=1)
ZopeTestCase.installProduct('PortalTransforms', quiet=1)
ZopeTestCase.installProduct('MimetypesRegistry', quiet=1)

ZopeTestCase.installProduct('summary')

from AccessControl import Unauthorized
from Products.PloneTestCase import PloneTestCase


PRODUCTS = ['summary']

PloneTestCase.setupPloneSite(products=PRODUCTS)

class TestSummaryField(PloneTestCase.PloneTestCase):

    def afterSetUp(self):
        """ Build a test infrastructure """
        
        self.loginAsPortalOwner()
        
        portal.portal_quickinstaller.installProduct("WNCDirectory")
        
        types = self.portal.portal_types.objectIds()        
        self.failUnless('SummarySampleContainer' in types, "Set INSTALL_DEMO_TYPES=True in config.py")        
                        
        self.portal.invokeFactory("SummarySampleContainer", "root")
        root = self.portal.root
        #root.initializeArchetype()
        
        for c in range(0, 3):
            root.invokeFactory("SummarySampleContent", "c" + str(c))
            content = root["c" + str(c)]
            
            content.setIntegerField(c)
            content.setTextField("Some text " + str(c))
            content.setFloatField(float(c) * 0.1)
            
        self.logout()
            
    def test_install(self):
        pass
    
    def test_get(self):
        """ See if summaryfield.get return proper values """
        
        root = self.portal.root
        data = root.getSummary()

        columns = data.getColumnIds()
        assert len(columns) == 3, "Got columns " + str(columns)

        assert "textField" in data.getColumnIds()
        assert "c2" in data.getRowIds(), "Data was:" + str(data)
            
        assert data.getValue("integerField", "c2") == 2, "c2 was:" + str(c2.data)
        assert data.getValue("textField", "c2") == "Some text 2", "c2 was:" + str(c2.data)
                        
    def test_set(self):
        """ Try set summary data """
        root = self.portal.root        
        data = root.getSummary()
        self.loginAsPortalOwner()
        root.setSummary(data)        
        
    def test_unauthorized(self):
        """ Try set summary without priviledges """
        try:
            root = self.portal.root        
            data = root.getSummary()
            root.setSummary(data)        
        except Unauthorized:
            return
        
        assert "Security breach"
                
        
    def test_lock(self):
        """ See if fields are propeply locked """
        pass
    
    def test_post(self):
        """ Simulate form HTTP post request """
        self.loginAsPortalOwner()
        root = self.portal.root   
                
        form = {}
        for c in range(0, 3):        
            
            name = "%s_%s_%s" % ("summary", "integerField", "c" + str(c))
            form[name] = 3
            
            name = "%s_%s_%s" % ("summary", "textField", "c" + str(c))
            form[name] = "oulu finland rules"
            
            name = "%s_%s_%s" % ("summary", "floatField", "c" + str(c))
            form[name] = "1.3"
    
        
        form["title"] = "Test title"
        form["description"] = "Blaa blaa blaa"
        
        
        #field = root.schema["summary"]
        #field.widget.process_form(root, field, form)
        
        class Moo:
            """ Fake REQUEST object """
            pass
            
        dummy = Moo()
        dummy.form = form
        
        errors = root.validate(REQUEST=dummy)
        print "got errors:" + str(errors)
        
        root._processForm(REQUEST=dummy)
            
        assert root.c1.getTextField() == "oulu finland rules", "got " + root.c1.getTextField()
        assert root.c2.getIntegerField() == 3, "got " + str(root.c0.getIntegerField())            
        
    def test_validation_fails(self):
        """ Summary validator fails """

        self.loginAsPortalOwner()
        root = self.portal.root   
                
        form = {}
        for c in range(0, 3):        
            
            name = "%s_%s_%s" % ("summary", "integerField", "c" + str(c))
            form[name] = 3
            
            name = "%s_%s_%s" % ("summary", "textField", "c" + str(c))
            form[name] = "oulu finland rules"
            
            # Put invalid data to float field
            name = "%s_%s_%s" % ("summary", "floatField", "c" + str(c))
            form[name] = "1.XXXX"
            
        form["title"] = "Test title"
        form["description"] = "Blaa blaa blaa"

        #field = root.schema["summary"]
        #field.widget.process_form(root, field, form)
        
        class Moo:
            """ Fake REQUEST object """
            pass
            
        dummy = Moo()
        dummy.form = form
                                                  
        errors = root.validate(REQUEST=dummy, data=True)
        
        assert len(errors) > 0, "Got errors: " + str(errors)
        
    def buildComplex(self):
        """ Build a complex demo type """
        self.loginAsPortalOwner()
                
        self.portal.invokeFactory("Patient", "patient")
        patient = self.portal.patient
        
        patient.invokeFactory("Treatment", "treatment")
        treatment = patient.treatment
        treatment.setDiseaseName("Flu")
        
        field = patient.schema["patientSummary"]
        assert not field.isValid(patient)
        
        treatment.invokeFactory("Visit", "visit1")
        visit1 = treatment.visit1

        visit1.setBloodPressure(120.8)
        visit1.setCost(2000)        
        visit1.setPatientAction("further")        
        
        assert field.isValid(patient)
                                
    def test_complex(self):
        """ See if our more complex use cases works 
        
        Note: this must be executed as last, since there is
        some uncleared set() between test_... methods() in 
        Plone 2.5.1
        """
        
        self.buildComplex()
        patient = self.portal.patient
        treatment = self.portal.patient.treatment
        field = patient.schema["patientSummary"]

        # Total column must appear only in the view mode
        data = patient.getPatientSummary(mode="view")        
        assert "total" in data.getColumnIds()
        assert len(data.getColumnIds()) == 2
        
        data = patient.getPatientSummary(mode="edit")        
        assert not("total" in data.getColumnIds()), "Got ids " + str(data.getColumnIds())
        assert len(data.getColumnIds()) == 1                

        data = patient.getPatientSummary(mode="process")                
        assert len(data.getColumnIds()) == 1                  
        patient.setPatientSummary(data)
        
        treatment.invokeFactory("Visit", "visit2")
        visit2 = treatment.visit2        
        visit2.setBloodPressure(113.1)
        visit2.setPatientAction("home")        
        visit2.setCost(4000)        
                        
        self.logout()
        data = patient.getPatientSummary(mode="view")
        total = data.getValue("total", "cost")
        assert total == 6000, "Total cost:" + str(total)
        
        assert field.isValid(patient)
        
    def test_selection_summary(self):
        """ See that selection field has correct value in view mode """
        self.buildComplex()
        patient = self.portal.patient
        treatment = self.portal.patient.treatment
        field = patient.schema["patientSummary"]
        
        data = self.portal.patient.getPatientSummary(mode="view")
        visit = self.portal.patient.treatment.visit1
        selectionSummarizer=data.getSummarizer(visit.getId(), "patientAction")  
        act = selectionSummarizer.getFriendlyValue(self.portal.patient, data, visit.getId(), "patientAction") 
        assert act == "Further laboratory samples are needed", "Got " + str(act)
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestSummaryField))
    return suite

if __name__ == '__main__':
    framework()

    